from django.contrib import admin

from swap_user.to_named_email.forms import NamedUserEmailForm
from swap_user.to_named_email.models import NamedEmailUser


class NamedEmailUserAdmin(admin.ModelAdmin):
    form = NamedUserEmailForm


admin.site.register(NamedEmailUser, NamedEmailUserAdmin)



from swap_user.forms import BaseUserForm
from swap_user.to_named_email.models import NamedEmailUser


class NamedUserEmailForm(BaseUserForm):
    class Meta(BaseUserForm.Meta):
        model = NamedEmailUser



from unittest.mock import MagicMock

from swap_user.to_named_email.forms import NamedUserEmailForm


def test_save_field():
    instance = MagicMock()
    data = {
        "email": "hello@world.com",
        "first_name": "Hello",
        "last_name": "World",
    }
    form = NamedUserEmailForm(data=data, instance=instance)

    assert form.is_valid() is True


def test_one_password():
    instance = MagicMock()
    data = {
        "email": "hello@world.com",
        "password_1": "Hello",
    }
    form = NamedUserEmailForm(data=data, instance=instance)

    assert form.is_valid() is False
    assert (
        form.errors.as_json()
        == '{"__all__": [{"message": "Provide both of passwords", "code": "provide_both_passwords"}]}'
    )


def test_not_matching_passwords():
    instance = MagicMock()
    data = {
        "email": "hello@world.com",
        "password_1": "Hello",
        "password_2": "World",
    }
    form = NamedUserEmailForm(data=data, instance=instance)

    assert form.is_valid() is False
    assert (
        form.errors.as_json()
        == '{"__all__": [{"message": "Passwords should be same", "code": "password_should_be_same"}]}'
    )


def test_matching_passwords():
    instance = MagicMock()
    data = {
        "email": "hello@world.com",
        "password_1": "Hello",
        "password_2": "Hello",
    }
    form = NamedUserEmailForm(data=data, instance=instance)

    assert form.is_valid() is True



# Generated by Django 2.2.16 on 2020-11-12 09:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("to_email", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="emailuser",
            options={"verbose_name": "email user", "verbose_name_plural": "email users"},
        ),
    ]



from django.conf import settings as django_settings
from django.utils.module_loading import import_string

IMPORT_SETTINGS = [
    "EMAIL_USER_ABSTRACT_BASE_CLASS",
]
DEFAULT_SETTINGS = {"EMAIL_USER_ABSTRACT_BASE_CLASS": "swap_user.to_email.models.AbstractEmailUser"}
NAMESPACE = "SWAP_USER"


class SwapUserSettings:
    def make_import(self, item, path):
        val = import_string(path)
        setattr(self, item, val)

        return val

    def __getattr__(self, item):
        try:
            namespaced_settings = getattr(django_settings, NAMESPACE)
            value = namespaced_settings[item]
        except (AttributeError, KeyError):
            value = DEFAULT_SETTINGS[item]

        if item in IMPORT_SETTINGS:
            value = self.make_import(item, value)

        return value


swap_user_settings = SwapUserSettings()



from django.db import models
from django.utils.translation import gettext_lazy as _

from swap_user.to_email.models import AbstractEmailUser


class AbstractNamedEmailUser(AbstractEmailUser):
    """
    Use this abstract class if you want to add a user with
    first name and last name.
    """

    first_name = models.CharField(
        verbose_name=_("first name"),
        max_length=50,
        # maybe remove
        blank=True,
    )
    last_name = models.CharField(
        verbose_name=_("last name"),
        max_length=100,
        # maybe remove
        blank=True,
    )

    class Meta:
        abstract = True

    def __str__(self):
        return f"{self.email} - {self.first_name} {self.last_name}"


class NamedEmailUser(AbstractNamedEmailUser):
    class Meta:
        swappable = "AUTH_USER_MODEL"
        verbose_name = "email user"
        verbose_name_plural = "email users"



from django import forms


class BaseUserForm(forms.ModelForm):
    password_1 = forms.CharField(
        label="Enter a new password", widget=forms.PasswordInput, required=False,
    )
    password_2 = forms.CharField(
        label="Repeat a new password", widget=forms.PasswordInput, required=False,
    )

    class Meta:
        exclude = ["password"]

    def clean(self):
        cleaned_data = super().clean()
        password_1 = cleaned_data["password_1"]
        password_2 = cleaned_data["password_2"]

        password_list = [password_1, password_2]
        not_empty_password_list = [item for item in password_list if item]

        # if one of passwords not filled - we are raising exception
        if len(not_empty_password_list) == 1:
            raise forms.ValidationError(
                message="Provide both of passwords", code="provide_both_passwords",
            )

        # if passwords doesn't match - we are raising exception
        if password_1 != password_2:
            raise forms.ValidationError(
                message="Passwords should be same", code="password_should_be_same",
            )

        return cleaned_data

    def save(self, commit=False):
        instance = super().save(commit)
        password_1 = self.cleaned_data["password_1"]

        if password_1:
            instance.set_password(password_1)

        instance.save()

        return instance



from django.contrib import admin

from swap_user.to_email.forms import EmailUserForm
from swap_user.to_email.models import EmailUser


class EmailUserAdmin(admin.ModelAdmin):
    form = EmailUserForm


admin.site.register(EmailUser, EmailUserAdmin)



# Generated by Django 2.2.15 on 2020-08-19 20:27

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="NamedEmailUser",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(blank=True, null=True, verbose_name="last login"),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "email",
                    models.EmailField(max_length=254, unique=True, verbose_name="email address"),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                        verbose_name="active",
                    ),
                ),
                (
                    "is_staff",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the user can log into this admin site.",
                        verbose_name="staff status",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(blank=True, max_length=50, verbose_name="first name"),
                ),
                (
                    "last_name",
                    models.CharField(blank=True, max_length=100, verbose_name="last name"),
                ),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={"swappable": "AUTH_USER_MODEL",},
        ),
    ]



from django.apps import AppConfig


class EmailUserConfig(AppConfig):
    # `name` for INSTALLED_APPS
    name = "swap_user.to_email"
    # `verbose name` for Admin panel display
    verbose_name = "Django Swap User"



import django


def pytest_configure():
    from django.conf import settings

    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
        MIDDLEWARE=(
            "django.middleware.common.CommonMiddleware",
            "django.contrib.sessions.middleware.SessionMiddleware",
            "django.contrib.auth.middleware.AuthenticationMiddleware",
            "django.contrib.messages.middleware.MessageMiddleware",
        ),
        INSTALLED_APPS=(
            "django.contrib.admin",
            "django.contrib.auth",
            "django.contrib.contenttypes",
            "django.contrib.sessions",
            "django.contrib.sites",
            "django.contrib.staticfiles",
            "swap_user",
        ),
    )

    django.setup()



from django.contrib.auth.models import BaseUserManager


class Manager(BaseUserManager):
    def _create(self, email, password, **extra_fields):
        if not email:
            raise ValueError("User should have email.")

        email = self.normalize_email(email)

        user = self.model(email=email, **extra_fields)

        user.set_password(password)
        user.save()

        return user

    def create_superuser(self, email, password, **extra_fields):
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_active", True)
        extra_fields.setdefault("is_superuser", True)

        return self._create(email, password, **extra_fields)

    def create_user(self, email, password, **extra_fields):
        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_active", False)
        extra_fields.setdefault("is_superuser", False)

        return self._create(email, password, **extra_fields)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['swap_user',
 'swap_user.managers',
 'swap_user.tests',
 'swap_user.to_email',
 'swap_user.to_email.migrations',
 'swap_user.to_named_email',
 'swap_user.to_named_email.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django>=2.2,<2.3']

setup_kwargs = {
    'name': 'django-swap-user',
    'version': '0.3.1',
    'description': '(Beta) Simple and flexible way to swap default Django User',
    'long_description': '# django-swap-user\n\n# архитектура\nприложение swap_user засплитовано на еще 2 приложения:\n  - email\n  - named_email\n  \nпотому что, если оставить их в одном аппе - то они вдвоем создают миграции и таблицы.\nесли их оставить, они будут считаться как 2 кастомные модели в пределах одного приложения, что вызывает\nнедоумение и когнитивную нагрузку.\n\nпри такой архитектуре (когда есть общий апп, который содержит внутренние аппы) - пользователь\nподключает только ту кастомную модель юзера, которая ему больше подходит.',
    'author': 'Artem Innokentiev',
    'author_email': 'artinnok@protonmail.com',
    'maintainer': 'Artem Innokentiev',
    'maintainer_email': 'artinnok@protonmail.com',
    'url': 'http://github.com/artinnok/django-swap-user',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from django.contrib.auth.base_user import AbstractBaseUser as DjangoAbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.db import models
from django.utils.translation import gettext_lazy as _

from swap_user.managers.email import Manager


class AbstractEmailUser(PermissionsMixin, DjangoAbstractBaseUser):
    """
    Abstract EmailUser implementation - subclass this class to provide your own
    custom User class with `email` field.

    Provides fields:
    - email
    - is_active (required by django.contrib.admin)
    - is_staff (required by django.contrib.admin)

    Provides attributes:
    - USERNAME_FIELD
    - EMAIL_FIELD
    - REQUIRED_FIELDS
    """

    email = models.EmailField(
        verbose_name=_("email address"),
        unique=True,
    )
    is_active = models.BooleanField(
        verbose_name=_("active"),
        default=False,
        help_text=_(
            "Designates whether this user should be treated as active. "
            "Unselect this instead of deleting accounts."
        ),
    )
    is_staff = models.BooleanField(
        verbose_name=_("staff status"),
        default=False,
        help_text=_("Designates whether the user can log into this admin site."),
    )

    objects = Manager()

    USERNAME_FIELD = "email"
    EMAIL_FIELD = "email"

    class Meta:
        abstract = True

    def __str__(self):
        return self.email

    clean = DjangoAbstractBaseUser.clean
    get_short_name = __str__
    get_full_name = __str__


class EmailUser(AbstractEmailUser):
    class Meta:
        swappable = "AUTH_USER_MODEL"
        verbose_name = "email user"
        verbose_name_plural = "email users"
